from abc import abstractmethod
from collections.abc import Iterable
from typing import Dict, Any, Optional

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.registry import registry


class BaseK8Check(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_entities: "Iterable[str]",
        guideline: Optional[str] = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type="k8",
            guideline=guideline
        )
        self.supported_specs = supported_entities
        registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        self.entity_type = entity_type
        return self.scan_spec_conf(conf)

    @abstractmethod
    def scan_spec_conf(self, conf: Dict[str, Any]) -> CheckResult:
        """Return result of Kubernetes object check."""
        raise NotImplementedError()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class SynapseWorkspaceAdministratorLoginPasswordHidden(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Azure Synapse Workspace administrator login password is not exposed"
        id = "CKV_AZURE_239"
        supported_resources = ['Microsoft.Synapse/workspaces']
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if conf.get("properties", {}).get("sqlAdministratorLoginPassword"):
            return CheckResult.FAILED
        return CheckResult.PASSED


check = SynapseWorkspaceAdministratorLoginPasswordHidden()



from typing import Any, Dict

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceCheck


class VMStorageOsDisk(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Virtual Machines use managed disks"
        id = "CKV_AZURE_92"
        supported_resources = ("Microsoft.Compute/virtualMachines",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get('properties')
        if not properties or not isinstance(properties, dict):
            return CheckResult.PASSED
        storage_profile = properties.get('storageProfile')
        if not storage_profile or not isinstance(storage_profile, dict):
            return CheckResult.PASSED
        os_disk = storage_profile.get('osDisk')
        data_disks = list(storage_profile.get('dataDisks', []))
        if os_disk and isinstance(os_disk, dict) and "vhd" in os_disk:
            self.evaluated_keys = ['os_disk']
            return CheckResult.FAILED
        if data_disks and any(isinstance(data_disk, dict) and "vhd" in data_disk for data_disk in data_disks):
            self.evaluated_keys = ['data_disks']
            return CheckResult.FAILED
        self.evaluated_keys = ['os_disk'] if os_disk else []
        if data_disks:
            self.evaluated_keys.append('data_disks')
        return CheckResult.PASSED


check = VMStorageOsDisk()



from typing import Any, List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class LBTargetGroup(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure AWS Load Balancer doesn't use HTTP protocol"
        id = "CKV_AWS_378"
        supported_resources = ('aws_lb_target_group', 'aws_alb_target_group',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'protocol'

    def get_forbidden_values(self) -> List[Any]:
        return ["HTTP"]


check = LBTargetGroup()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class ConnectInstanceS3StorageConfigUsesCMK(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Connect Instance S3 Storage Config uses CMK"
        id = "CKV_AWS_270"
        supported_resources = ("aws_connect_instance_storage_config",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "storage_config/[0]/s3_config/[0]/encryption_config/[0]/key_id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = ConnectInstanceS3StorageConfigUsesCMK()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ACREnableRetentionPolicy(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure a retention policy is set to cleanup untagged manifests."
        id = "CKV_AZURE_167"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if 'retention_policy_in_days' in conf:
            return CheckResult.PASSED

        if 'retention_policy' in conf:
            retention_policy = conf['retention_policy'][0]
            if isinstance(retention_policy, dict) and retention_policy.get('enabled') == [True]:
                return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['retention_policy_in_days', 'retention_policy/enabled']


check = ACREnableRetentionPolicy()



from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class SSMParameterUsesCMK(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure SSM parameters are using KMS CMK"
        id = "CKV_AWS_337"
        supported_resources = ("aws_ssm_parameter",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("type")[0] != "SecureString":
            return CheckResult.PASSED
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "key_id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = SSMParameterUsesCMK()



from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class SagemakerDataQualityJobDefinitionVolumeEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Amazon Sagemaker Data Quality Job uses KMS to encrypt data on attached storage volume"
        id = "CKV_AWS_368"
        supported_resources = ['aws_sagemaker_data_quality_job_definition']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "job_resources/[0]/cluster_config/[0]/volume_kms_key_id"

    def get_expected_value(self):
        return ANY_VALUE


check = SagemakerDataQualityJobDefinitionVolumeEncryption()



from __future__ import annotations

import json
import logging
import os
import platform
import threading
from collections import defaultdict
from pathlib import Path
from typing import Optional, Dict, Mapping, Set, Tuple, Callable, Any, List, cast, TYPE_CHECKING, overload, TextIO, Type

import hcl2

from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import filter_ignored_paths, IGNORE_HIDDEN_DIRECTORY_ENV
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR, RESOLVED_MODULE_ENTRY_NAME
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.deep_merge import pickle_deep_merge
from checkov.common.util.env_vars_config import env_vars_config
from checkov.common.util.stopit import ThreadingTimeout, SignalTimeout
from checkov.common.util.stopit.utils import BaseTimeout
from checkov.common.util.type_forcers import force_list
from checkov.common.variables.context import EvaluationContext
from checkov.terraform import validate_malformed_definitions, clean_bad_definitions
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.module import Module
from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.module_finder import load_tf_modules
from checkov.terraform.module_loading.registry import module_loader_registry as default_ml_registry, \
    ModuleLoaderRegistry
from checkov.common.util.parser_utils import is_acceptable_module_param
from checkov.terraform.modules.module_utils import safe_index, \
    remove_module_dependency_from_path, \
    clean_parser_types, serialize_definitions, _Hcl2Payload
from checkov.terraform.modules.module_objects import TFModule, TFDefinitionKey

if TYPE_CHECKING:
    from typing_extensions import TypeGuard


def _filter_ignored_paths(root: str, paths: list[str], excluded_paths: list[str] | None) -> None:
    filter_ignored_paths(root, paths, excluded_paths)
    for path in force_list(paths):
        if path == default_ml_registry.external_modules_folder_name:
            paths.remove(path)


class TFParser:
    def __init__(self, module_class: type[Module] = Module) -> None:
        self.module_class = module_class
        self._parsed_directories: set[str] = set()
        self.external_modules_source_map: Dict[Tuple[str, str], str] = {}
        self.module_address_map: Dict[Tuple[str, str], str] = {}
        self.loaded_files_map: dict[str, dict[str, list[dict[str, Any]]] | None] = {}
        self.external_variables_data: list[tuple[str, Any, str]] = []
        self.temp_tf_definition: dict[str, Any] = {}

    def _init(self, directory: str,
              out_evaluations_context: Dict[TFDefinitionKey, Dict[str, EvaluationContext]] | None,
              out_parsing_errors: Dict[str, Exception] | None,
              env_vars: Mapping[str, str] | None,
              download_external_modules: bool,
              external_modules_download_path: str,
              excluded_paths: Optional[List[str]] = None,
              tf_var_files: Optional[List[str]] = None) -> None:
        self.directory = directory
        self.out_definitions: dict[TFDefinitionKey, dict[str, list[dict[str, Any]]]] = {}
        self.out_evaluations_context = {} if out_evaluations_context is None else out_evaluations_context
        self.out_parsing_errors = {} if out_parsing_errors is None else out_parsing_errors
        self.env_vars = dict(os.environ) if env_vars is None else env_vars
        self.download_external_modules = download_external_modules
        self.external_modules_download_path = external_modules_download_path
        self.external_modules_source_map = {}
        self.module_address_map = {}
        self.tf_var_files = tf_var_files
        self.dirname_cache: dict[str, str] = {}
        self.excluded_paths = excluded_paths
        self.visited_definition_keys: set[TFDefinitionKey] = set()
        self.module_to_resolved: dict[tuple[TFDefinitionKey | None, str], list[TFDefinitionKey]] = {}
        self.keys_to_remove: set[TFDefinitionKey] = set()

    def _check_process_dir(self, directory: str) -> bool:
        if directory not in self._parsed_directories:
            self._parsed_directories.add(directory)
            return True
        else:
            return False

    def parse_directory(
            self,
            directory: str,
            out_evaluations_context: Dict[TFDefinitionKey, Dict[str, EvaluationContext]] | None = None,
            out_parsing_errors: Dict[str, Exception] | None = None,
            env_vars: Mapping[str, str] | None = None,
            download_external_modules: bool = False,
            external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
            excluded_paths: Optional[List[str]] = None,
            vars_files: Optional[List[str]] = None,
            external_modules_content_cache: Optional[Dict[str, ModuleContent | None]] = None,
    ) -> dict[TFDefinitionKey, dict[str, list[dict[str, Any]]]]:
        self._init(directory, out_evaluations_context, out_parsing_errors, env_vars,
                   download_external_modules, external_modules_download_path, excluded_paths)
        self._parsed_directories.clear()
        default_ml_registry.root_dir = directory
        default_ml_registry.download_external_modules = download_external_modules
        default_ml_registry.external_modules_folder_name = external_modules_download_path
        default_ml_registry.module_content_cache = external_modules_content_cache if external_modules_content_cache else {}
        load_tf_modules(directory)
        self._parse_directory(dir_filter=lambda d: self._check_process_dir(d), vars_files=vars_files)
        self._update_resolved_modules()
        return self.out_definitions

    def parse_file(self, file: str, parsing_errors: dict[str, Exception]) -> Optional[Dict[str, Any]]:
        if file.endswith(".tf") or file.endswith(".tf.json") or file.endswith(".hcl"):
            parse_result = load_or_die_quietly(file, parsing_errors)
            if parse_result:
                parse_result = serialize_definitions(parse_result)
                parse_result = clean_parser_types(parse_result)
                return parse_result

        return None

    def _parse_directory(self, include_sub_dirs: bool = True,
                         module_loader_registry: ModuleLoaderRegistry = default_ml_registry,
                         dir_filter: Callable[[str], bool] = lambda _: True,
                         vars_files: Optional[List[str]] = None) -> None:

        keys_referenced_as_modules: set[TFDefinitionKey] = set()

        if include_sub_dirs:
            for sub_dir, d_names, _ in os.walk(self.directory):
                _filter_ignored_paths(sub_dir, d_names, self.excluded_paths)
                if dir_filter(os.path.abspath(sub_dir)):
                    self._internal_dir_load(sub_dir, module_loader_registry, dir_filter,
                                            keys_referenced_as_modules, vars_files=vars_files,
                                            excluded_paths=self.excluded_paths)
        else:
            self._internal_dir_load(self.directory, module_loader_registry, dir_filter,
                                    keys_referenced_as_modules, vars_files=vars_files)

        for key in keys_referenced_as_modules:
            if key in self.out_definitions:
                del self.out_definitions[key]

    def _internal_dir_load(
            self,
            directory: str,
            module_loader_registry: ModuleLoaderRegistry,
            dir_filter: Callable[[str], bool],
            keys_referenced_as_modules: set[TFDefinitionKey],
            specified_vars: Optional[Mapping[str, str]] = None,
            vars_files: Optional[List[str]] = None,
            excluded_paths: Optional[List[str]] = None,
            nested_modules_data: dict[str, Any] | None = None,
    ) -> None:

        dir_contents = list(os.scandir(directory))
        if excluded_paths or IGNORE_HIDDEN_DIRECTORY_ENV:
            filter_ignored_paths(directory, dir_contents, excluded_paths)

        tf_files_to_load = self.handle_variables(dir_contents, vars_files, specified_vars)
        files_to_data = self._load_files(tf_files_to_load)
        for file, data in sorted(files_to_data, key=lambda x: x[0]):
            if not data:
                continue
            self.out_definitions[TFDefinitionKey(file)] = data
            self.add_external_vars_from_data(data, file)

        force_final_module_load = False
        for i in range(0, 10):
            logging.debug(f"Module load loop {i}")
            dir_filter(directory)
            has_more_modules = self._load_modules(
                directory, module_loader_registry, dir_filter,
                keys_referenced_as_modules, force_final_module_load,
                nested_modules_data=nested_modules_data
            )
            made_var_changes = False
            if not has_more_modules:
                break
            elif not made_var_changes:
                force_final_module_load = True

    def _load_files(
            self,
            files: list[os.DirEntry[str]],
    ) -> list[tuple[str, dict[str, list[dict[str, Any]]] | None]]:
        def _load_file(
                file: os.DirEntry[str]
        ) -> tuple[tuple[str, dict[str, list[dict[str, Any]]] | None], dict[str, Exception]]:
            parsing_errors: dict[str, Exception] = {}
            result = load_or_die_quietly(file, parsing_errors)
            for path, e in parsing_errors.items():
                parsing_errors[path] = e

            return (file.path, result), parsing_errors

        files_to_data: list[tuple[str, dict[str, list[dict[str, Any]]] | None]] = []
        files_to_parse = []
        for file in files:
            data = self.loaded_files_map.get(file.path)
            if data:
                files_to_data.append((file.path, data))
            else:
                files_to_parse.append(file)

        results = [_load_file(f) for f in files_to_parse]
        for result, parsing_errors in results:
            self.out_parsing_errors.update(parsing_errors)
            files_to_data.append(result)
            if result[0] not in self.loaded_files_map:
                self.loaded_files_map[result[0]] = result[1]
        return files_to_data

    def _load_modules(self, root_dir: str, module_loader_registry: ModuleLoaderRegistry,
                      dir_filter: Callable[[str], bool],
                      keys_referenced_as_modules: Set[TFDefinitionKey], ignore_unresolved_params: bool = False,
                      nested_modules_data: dict[str, Any] | None = None) -> bool:
        all_module_definitions: dict[TFDefinitionKey, dict[str, list[dict[str, Any]]]] = {}
        skipped_a_module = False
        for file in list(self.out_definitions.keys()):
            if not self.should_loaded_file(file, root_dir):
                continue

            #  Dont run over the nested because we already run on them - dont remove.
            if file.tf_source_modules:
                continue

            file_data = self.out_definitions.get(file)
            if file_data is None:
                continue
            module_calls = file_data.get("module")
            if not module_calls or not isinstance(module_calls, list):
                continue

            for module_call in module_calls:
                if not isinstance(module_call, dict):
                    continue

                for module_call_name, module_call_data in module_call.items():
                    if not isinstance(module_call_data, dict):
                        continue

                    file_key = self.get_file_key_with_nested_data(file, nested_modules_data)
                    current_nested_data = (file_key, module_call_name)
                    resolved_loc_list = []
                    if current_nested_data in self.module_to_resolved:
                        resolved_loc_list = self.module_to_resolved[current_nested_data]
                    self.module_to_resolved[current_nested_data] = resolved_loc_list

                    specified_vars = {
                        k: v[0] if isinstance(v, list) and v else v
                        for k, v in module_call_data.items()
                        if k != "source" and k != "version"
                    }
                    skip_module = self.should_skip_a_module(specified_vars, ignore_unresolved_params)
                    if skip_module:
                        # keep module skip info till the end
                        skipped_a_module = True
                        continue

                    version = self.get_module_version(module_call_data)
                    source = self.get_module_source(module_call_data, module_call_name, file)
                    if not source:
                        continue

                    try:
                        content_path = self.get_content_path(module_loader_registry, root_dir, source, version)
                        if not content_path:
                            continue
                        new_nested_modules_data = {'module_name': module_call_name, 'file': file,
                                                   'nested_modules_data': nested_modules_data}
                        self._internal_dir_load(
                            directory=content_path,
                            module_loader_registry=module_loader_registry,
                            dir_filter=dir_filter, specified_vars=specified_vars,
                            keys_referenced_as_modules=keys_referenced_as_modules,
                            nested_modules_data=new_nested_modules_data
                        )

                        module_definitions = {
                            path: definition
                            for path, definition in self.out_definitions.items()
                            if self.get_dirname(path) == content_path and not path.tf_source_modules
                        }
                        if not module_definitions:
                            continue

                        keys = list(module_definitions.keys())
                        for key in keys:
                            if not self.should_process_key(key, file):
                                continue
                            keys_referenced_as_modules.add(key)
                            new_key = self.get_new_nested_module_key(key, file, module_call_name, nested_modules_data)
                            if new_key in self.visited_definition_keys:
                                del module_definitions[key]
                                del self.out_definitions[key]
                                continue

                            module_definitions[new_key] = module_definitions[key]
                            del module_definitions[key]
                            del self.out_definitions[key]
                            self.keys_to_remove.add(key)

                            self.visited_definition_keys.add(new_key)
                            if new_key not in resolved_loc_list:
                                resolved_loc_list.append(new_key)

                        if all_module_definitions:
                            pickle_deep_merge(all_module_definitions, module_definitions)
                        else:
                            all_module_definitions = module_definitions

                        self.external_modules_source_map[(source, version)] = content_path
                    except Exception as e:
                        logging.warning(
                            f"Unable to load module - source: {source}, version: {version}, error: {str(e)}")

        if all_module_definitions:
            pickle_deep_merge(self.out_definitions, all_module_definitions)
        return skipped_a_module

    def parse_hcl_module(
            self,
            source_dir: str,
            source: str,
            download_external_modules: bool = False,
            external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
            parsing_errors: dict[str, Exception] | None = None,
            excluded_paths: list[str] | None = None,
            vars_files: list[str] | None = None,
            external_modules_content_cache: dict[str, ModuleContent | None] | None = None,
    ) -> tuple[Module, dict[TFDefinitionKey, dict[str, Any]]]:
        tf_definitions = self.parse_directory(
            directory=source_dir, out_evaluations_context={},
            out_parsing_errors=parsing_errors if parsing_errors is not None else {},
            download_external_modules=download_external_modules,
            external_modules_download_path=external_modules_download_path, excluded_paths=excluded_paths,
            vars_files=vars_files, external_modules_content_cache=external_modules_content_cache
        )
        tf_definitions = clean_parser_types(tf_definitions)
        tf_definitions = serialize_definitions(tf_definitions)

        module, tf_definitions = self.parse_hcl_module_from_tf_definitions(tf_definitions, source_dir, source)

        return module, tf_definitions

    def parse_multi_graph_hcl_module(
            self,
            source_dir: str,
            source: str,
            download_external_modules: bool = False,
            external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
            parsing_errors: dict[str, Exception] | None = None,
            excluded_paths: list[str] | None = None,
            vars_files: list[str] | None = None,
            external_modules_content_cache: dict[str, ModuleContent | None] | None = None,
    ) -> list[tuple[Module, list[dict[TFDefinitionKey, dict[str, Any]]]]]:
        """
        This function is similar to parse_hcl_module, except that it creates a list of tuples instead of a single tuple.
        The objective is to create a collection of TF definitions based on directory, instead of a single big structure.
        This will allow us to boost performance by running on several smaller objects rather than a single one.
        """
        tf_definitions = self.parse_directory(
            directory=source_dir, out_evaluations_context={},
            out_parsing_errors=parsing_errors if parsing_errors is not None else {},
            download_external_modules=download_external_modules,
            external_modules_download_path=external_modules_download_path, excluded_paths=excluded_paths,
            vars_files=vars_files, external_modules_content_cache=external_modules_content_cache
        )
        tf_definitions = clean_parser_types(tf_definitions)
        tf_definitions = serialize_definitions(tf_definitions)

        dirs_to_definitions = self.create_definition_by_dirs(tf_definitions)

        definitions_dir_and_source_iterable = [(definitions, source_path, source) for source_path, definitions in
                                               dirs_to_definitions.items()]
        modules_and_definitions_tuple: list[tuple[Module, list[dict[TFDefinitionKey, dict[str, Any]]]]] = \
            list(parallel_runner.run_function(self.parse_hcl_module_from_multi_tf_definitions,
                                              definitions_dir_and_source_iterable))

        return modules_and_definitions_tuple

    def create_definition_by_dirs(self, tf_definitions: dict[TFDefinitionKey, dict[str, list[dict[str, Any]]]]
                                  ) -> dict[str, list[dict[TFDefinitionKey, dict[str, Any]]]]:
        dirs_to_definitions: dict[str, list[dict[TFDefinitionKey, dict[str, Any]]]] = defaultdict(list)
        for tf_definition_key, tf_value in tf_definitions.items():
            source_module = tf_definition_key.tf_source_modules
            if source_module is None:
                # No module - add new entry to dirs_to_definitions with the path as key
                dir_path = os.path.dirname(tf_definition_key.file_path)
                dirs_to_definitions[dir_path].append({tf_definition_key: tf_value})
            else:
                # iterate over nested modules while adding directories on the way
                while source_module is not None:
                    if source_module.nested_tf_module is None:
                        dir_path = os.path.dirname(source_module.path)
                        dirs_to_definitions[dir_path].append({tf_definition_key: tf_value})
                    source_module = source_module.nested_tf_module
        return dirs_to_definitions

    def _remove_unused_path_recursive(self, path: TFDefinitionKey) -> None:
        self.out_definitions.pop(path, None)
        for key in list(self.module_to_resolved.keys()):
            file_key = None
            if isinstance(key[0], TFDefinitionKey):
                file_key = key[0]
            elif key[0] is not None:
                file_key, module_index, module_name = key
            if path == file_key:
                for resolved_path in self.module_to_resolved[key]:
                    self._remove_unused_path_recursive(resolved_path)
                self.module_to_resolved.pop(key, None)

    def _update_resolved_modules(self) -> None:
        for key in list(self.module_to_resolved.keys()):
            file_key, module_name = key
            if file_key in self.keys_to_remove:
                for path in self.module_to_resolved[key]:
                    self._remove_unused_path_recursive(path)
                self.module_to_resolved.pop(key, None)

        for key, resolved_list in self.module_to_resolved.items():
            file_key, module_name = key
            if file_key not in self.out_definitions:
                continue

            idx = self.get_idx_by_module_name(self.out_definitions[file_key]['module'], module_name)
            if idx is None:
                continue

            self.out_definitions[file_key]['module'][idx][module_name][RESOLVED_MODULE_ENTRY_NAME] = resolved_list

    @staticmethod
    def get_idx_by_module_name(module_data_list: list[dict[str, Any]], module_name: str) -> int | None:
        for idx, module_data in enumerate(module_data_list):
            if module_name in module_data:
                return idx

        return None

    @overload
    def parse_hcl_module_from_tf_definitions(
            self,
            tf_definitions: dict[str, dict[str, Any]],
            source_dir: str,
            source: str,
    ) -> tuple[Module, dict[str, dict[str, Any]]]:
        ...

    @overload
    def parse_hcl_module_from_tf_definitions(
            self,
            tf_definitions: dict[TFDefinitionKey, dict[str, Any]],
            source_dir: str,
            source: str,
    ) -> tuple[Module, dict[TFDefinitionKey, dict[str, Any]]]:
        ...

    def parse_hcl_module_from_tf_definitions(
            self,
            tf_definitions: dict[str, dict[str, Any]] | dict[TFDefinitionKey, dict[str, Any]],
            source_dir: str,
            source: str,
    ) -> tuple[Module, dict[str, dict[str, Any]] | dict[TFDefinitionKey, dict[str, Any]]]:
        module = self.get_new_module(
            source_dir=source_dir,
            external_modules_source_map=self.external_modules_source_map,
        )
        self.add_tfvars(module, source)
        copy_of_tf_definitions = pickle_deepcopy(tf_definitions)
        module.temp_tf_definition = tf_definitions  # type:ignore  # will be TFDefinitionKey and not string
        for file_path, blocks in copy_of_tf_definitions.items():
            for block_type in blocks:
                try:
                    module.add_blocks(block_type, blocks[block_type], file_path, source)
                except Exception as e:
                    logging.warning(f'Failed to add block {blocks[block_type]}. Error:')
                    logging.warning(e, exc_info=False)
        return module, tf_definitions

    def parse_hcl_module_from_multi_tf_definitions(
            self,
            tf_definitions: list[dict[TFDefinitionKey, dict[str, Any]]],
            source_dir: str,
            source: str,
    ) -> tuple[Module, list[dict[TFDefinitionKey, dict[str, Any]]]]:
        module = self.get_new_module(
            source_dir=source_dir,
            external_modules_source_map=self.external_modules_source_map,
        )
        self.add_tfvars_with_source_dir(module, source, source_dir)
        copy_of_tf_definitions = pickle_deepcopy(tf_definitions)
        for tf_def in copy_of_tf_definitions:
            for file_path, blocks in tf_def.items():
                for block_type in blocks:
                    try:
                        module.add_blocks(block_type, blocks[block_type], file_path, source)
                    except Exception as e:
                        logging.warning(f'Failed to add block {blocks[block_type]}. Error:')
                        logging.warning(e, exc_info=False)
        return module, tf_definitions

    def get_file_key_with_nested_data(
            self, file: TFDefinitionKey | None, nested_data: dict[str, Any] | None
    ) -> TFDefinitionKey | None:
        if not nested_data or file is None:
            return file
        nested_str = self.get_file_key_with_nested_data(nested_data.get("file"), nested_data.get('nested_modules_data'))
        nested_module_name = nested_data.get('module_name')
        return get_tf_definition_object_from_module_dependency(file, nested_str, nested_module_name)

    def get_new_nested_module_key(
            self, key: TFDefinitionKey, file: TFDefinitionKey | None, module_name: str | None,
            nested_data: Optional[dict[str, Any]]
    ) -> TFDefinitionKey:
        if not nested_data or not file:
            return get_tf_definition_object_from_module_dependency(key, file, module_name)
        visited_key_to_add = get_tf_definition_object_from_module_dependency(key, file, module_name)
        self.visited_definition_keys.add(visited_key_to_add)
        nested_key = self.get_new_nested_module_key(file, nested_data.get('file'),
                                                    nested_data.get('module_name'),
                                                    nested_data.get('nested_modules_data'))
        return get_tf_definition_object_from_module_dependency(key, nested_key, module_name)

    def add_tfvars(self, module: Module, source: str) -> None:
        if not self.external_variables_data:
            return
        for (var_name, default, path) in self.external_variables_data:
            if ".tfvars" in path:
                block = [{var_name: {"default": default}}]
                module.add_blocks(BlockType.TF_VARIABLE, block, path, source)

    def add_tfvars_with_source_dir(self, module: Module, source: str, source_dir: str) -> None:
        if not self.external_variables_data:
            return
        for var_name, default, path in self.external_variables_data:
            if ".tfvars" in path:
                if Path(source_dir) in Path(path).parents:
                    block = [{var_name: {"default": default}}]
                    module.add_blocks(BlockType.TF_VARIABLE, block, path, source)

    def get_dirname(self, path: TFDefinitionKey) -> str:
        file_path = path.file_path
        dirname_path = self.dirname_cache.get(file_path)
        if not dirname_path:
            dirname_path = os.path.dirname(file_path)
            self.dirname_cache[file_path] = dirname_path
        return dirname_path

    def should_loaded_file(self, file: TFDefinitionKey, root_dir: str) -> bool:
        return not self.get_dirname(file) != root_dir

    def get_module_source(
            self, module_call_data: dict[str, Any], module_call_name: str, file: TFDefinitionKey
    ) -> Optional[str]:
        source = module_call_data.get("source")
        if not source or not isinstance(source, list):
            return None
        source = source[0]
        if not self.is_valid_source(source, module_call_name):
            return None

        if source.startswith("./") or source.startswith("../"):
            file_to_load = file.file_path
            source = os.path.normpath(
                os.path.join(os.path.dirname(remove_module_dependency_from_path(file_to_load)), source))
        return source

    def add_external_vars_from_data(self, data: dict[str, Any], file: str) -> None:
        var_blocks = data.get("variable")
        if var_blocks and isinstance(var_blocks, list):
            for var_block in var_blocks:
                if not isinstance(var_block, dict):
                    continue
                for var_name, var_definition in var_block.items():
                    if not isinstance(var_definition, dict):
                        continue

                    default_value = var_definition.get("default")
                    if default_value is not None and isinstance(default_value, list):
                        self.external_variables_data.append((var_name, default_value[0], file))

    def handle_variables(
            self,
            dir_contents: list[os.DirEntry[str]],
            vars_files: None | list[str],
            specified_vars: Mapping[str, str] | None,
    ) -> list[os.DirEntry[str]]:
        tf_files_to_load = []
        hcl_tfvars: Optional[os.DirEntry[str]] = None
        json_tfvars: Optional[os.DirEntry[str]] = None
        auto_vars_files: List[os.DirEntry[str]] = []
        explicit_var_files: List[os.DirEntry[str]] = []
        for file in dir_contents:
            try:
                if not file.is_file():
                    continue
            except OSError:
                continue

            if file.name == "terraform.tfvars.json":
                json_tfvars = file
            elif file.name == "terraform.tfvars":
                hcl_tfvars = file
            elif file.name.endswith(".auto.tfvars.json") or file.name.endswith(".auto.tfvars"):
                auto_vars_files.append(file)
            elif vars_files and file.path in vars_files:
                explicit_var_files.append(file)
            elif file.name.endswith(".tf") or file.name.endswith('.hcl'):  # TODO: add support for .tf.json
                tf_files_to_load.append(file)

        for key, value in self.env_vars.items():
            if not key.startswith("TF_VAR_"):
                continue
            self.external_variables_data.append((key[7:], value, f"env:{key}"))
        if hcl_tfvars:  # terraform.tfvars
            data = load_or_die_quietly(hcl_tfvars, self.out_parsing_errors, clean_definitions=False)
            if data:
                self.external_variables_data.extend([(k, safe_index(v, 0), hcl_tfvars.path) for k, v in data.items()])
        if json_tfvars:  # terraform.tfvars.json
            data = load_or_die_quietly(json_tfvars, self.out_parsing_errors)
            if data:
                self.external_variables_data.extend([(k, v, json_tfvars.path) for k, v in data.items()])

        auto_var_files_to_data = self._load_files(auto_vars_files)
        for var_file, data in sorted(auto_var_files_to_data, key=lambda x: x[0]):
            if data:
                self.external_variables_data.extend([(k, v, var_file) for k, v in data.items()])

        explicit_var_files_to_data = self._load_files(explicit_var_files)
        # it's possible that os.scandir returned the var files in a different order than they were specified
        if vars_files:
            for var_file, data in sorted(explicit_var_files_to_data, key=lambda x: vars_files.index(x[0])):
                if data:
                    self.external_variables_data.extend([(k, v, var_file) for k, v in data.items()])

        if specified_vars:  # specified
            self.external_variables_data.extend([(k, v, "manual specification") for k, v in specified_vars.items()])

        return tf_files_to_load

    @staticmethod
    def get_module_version(module_call_data: dict[str, Any]) -> str:
        version = module_call_data.get("version", "latest")
        if version and isinstance(version, list):
            version = version[0]
        return cast(str, version)

    @staticmethod
    def should_process_key(key: TFDefinitionKey, file: TFDefinitionKey) -> bool:
        return bool(not key.tf_source_modules or file.tf_source_modules)

    @staticmethod
    def is_valid_source(source: Any, module_call_name: str) -> TypeGuard[str]:
        if not isinstance(source, str):
            logging.debug(f"Skipping loading of {module_call_name} as source is not a string, it is: {source}")
            return False
        elif source in ['./', '.']:
            logging.debug(f"Skipping loading of {module_call_name} as source is the current dir")
            return False
        return True

    @staticmethod
    def should_skip_a_module(specified_vars: dict[str, Any], ignore_unresolved_params: bool) -> bool:
        if not ignore_unresolved_params:
            has_unresolved_params = False
            for k, v in specified_vars.items():
                if not is_acceptable_module_param(v) or not is_acceptable_module_param(k):
                    has_unresolved_params = True
                    break
            if has_unresolved_params:
                return True
        return False

    @staticmethod
    def get_content_path(module_loader_registry: ModuleLoaderRegistry, root_dir: str, source: str, version: str) -> \
            Optional[str]:
        content = module_loader_registry.load(root_dir, source, version)
        if not content or not content.loaded():
            logging.info(f'Got no content for {source}:{version}')
            return None
        return content.path()

    @staticmethod
    def get_new_module(
            source_dir: str,
            external_modules_source_map: dict[tuple[str, str], str],
    ) -> Module:
        return Module(
            source_dir=source_dir,
            external_modules_source_map=external_modules_source_map,
        )


def is_nested_object(full_path: TFDefinitionKey) -> bool:
    return True if full_path.tf_source_modules else False


def get_tf_definition_object_from_module_dependency(
        path: TFDefinitionKey, module_dependency: TFDefinitionKey | None, module_dependency_name: str | None
) -> TFDefinitionKey:
    if not module_dependency:
        return path
    if not is_nested_object(module_dependency):
        return TFDefinitionKey(path.file_path, TFModule(path=module_dependency.file_path, name=module_dependency_name))
    return TFDefinitionKey(path.file_path, TFModule(path=module_dependency.file_path, name=module_dependency_name,
                                                    nested_tf_module=module_dependency.tf_source_modules))


def load_or_die_quietly(
        file: str | Path | os.DirEntry[str], parsing_errors: dict[str, Exception], clean_definitions: bool = True
) -> Optional[_Hcl2Payload]:
    """
    Load JSON or HCL, depending on filename.
    :return: None if the file can't be loaded
    """
    file_path = os.fspath(file)
    file_name = os.path.basename(file_path)

    if file_name.endswith('.tfvars'):
        clean_definitions = False

    try:
        logging.debug(f"Parsing {file_path}")

        with open(file_path, "r", encoding="utf-8-sig") as f:
            if file_name.endswith(".json"):
                return cast("_Hcl2Payload", json.load(f))
            else:
                raw_data = __parse_with_timeout(f)
                non_malformed_definitions = validate_malformed_definitions(raw_data)
                if clean_definitions:
                    return clean_bad_definitions(non_malformed_definitions)
                else:
                    return non_malformed_definitions
    except Exception as e:
        logging.debug(f'failed while parsing file {file_path}', exc_info=True)
        parsing_errors[file_path] = e
        return None


# if we are not running in a thread, run the hcl2.load function with a timeout, to prevent from getting stuck in parsing.
def __parse_with_timeout(f: TextIO) -> dict[str, list[dict[str, Any]]]:
    # setting up timeout class
    timeout_class: Optional[Type[BaseTimeout]] = None
    if platform.system() == 'Windows':
        timeout_class = ThreadingTimeout
    elif threading.current_thread() is threading.main_thread():
        timeout_class = SignalTimeout

    # if we're not running on the main thread, don't use timeout
    parsing_timeout = env_vars_config.HCL_PARSE_TIMEOUT_SEC or 0
    if not timeout_class or not parsing_timeout:
        return hcl2.load(f)

    with timeout_class(parsing_timeout) as to_ctx_mgr:
        raw_data = hcl2.load(f)
    if to_ctx_mgr.state == to_ctx_mgr.TIMED_OUT:
        logging.debug(f"reached timeout when parsing file {f} using hcl2")
        raise Exception(f"file took more than {parsing_timeout} seconds to parse")
    return raw_data



from datetime import datetime

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class TransferServerLatestPolicy(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AWS Transfer Server uses latest Security Policy"
        id = "CKV_AWS_380"
        supported_resources = ('aws_transfer_server',)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def __check_policy_date(self, policy_string: str) -> bool:
        # Extract the year and month from the policy string
        # We assume that the year and month appear in the format 'YYYY-MM'

        # Split the string based on the '-' separator
        parts = policy_string.split('-')

        # Loop through the parts and check for the first valid year-month pair
        for i in range(len(parts) - 1):
            # Try to form a year-month date from consecutive parts
            year = parts[i]
            month = parts[i + 1]

            try:
                # If both year and month are integers and valid, create the date
                policy_date = datetime(int(year), int(month), 1)
                break
            except ValueError:
                continue
        else:
            # If no valid year-month combination is found, raise an error
            raise ValueError("No valid date found in the policy string.")

        # Get the current date
        current_date = datetime.now()

        # Calculate the time difference in months
        years_diff = current_date.year - policy_date.year
        months_diff = current_date.month - policy_date.month

        total_months_diff = years_diff * 12 + months_diff

        # If the difference is more than or equal to 24 months, return False
        return total_months_diff < 24

    def scan_resource_conf(self, conf: any) -> CheckResult:
        """
        Makes sure the Security Policy is no older than 2 years
        """
        security_policy = conf.get('security_policy_name')
        if security_policy:
            if self.__check_policy_date(security_policy[0]):
                return CheckResult.PASSED
        return CheckResult.FAILED  # default is TransferSecurityPolicy-2018-11 which is old: https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/transfer_server

    def get_evaluated_key(self) -> str:
        return "security_policy_name"


check = TransferServerLatestPolicy()



from enum import Enum


SELF_REFERENCE = "__self__"


class GraphSource(str, Enum):
    ANSIBLE = "Ansible"
    ARM = "ARM"
    BICEP = "Bicep"
    CLOUDFORMATION = "CloudFormation"
    DOCKERFILE = "Dockerfile"
    GITHUB_ACTIONS = "GitHubActions"
    KUBERNETES = "Kubernetes"
    TERRAFORM = "Terraform"
    TERRAFORM_PLAN = "terraform_plan"
    KUSTOMIZE = "kustomize"
    GITHUB_ACTION = "github_actions"
    HELM = "helm"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value



from typing import Any, List
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AppServicePHPVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that 'PHP version' is the latest, if used to run the web app"
        id = "CKV_AZURE_81"
        supported_resources = ["Microsoft.Web/sites"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.UNKNOWN)

    def get_inspected_key(self) -> str:
        return "properties/siteConfig/phpVersion"

    def get_expected_values(self) -> List[Any]:
        return ["8.1", "8.2"]


check = AppServicePHPVersion()



from __future__ import annotations

from typing import Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class FunctionAppsEnableAuthentication(BaseResourceCheck):

    def __init__(self) -> None:
        name = "Ensure that function apps enables Authentication"
        id = "CKV_AZURE_56"
        supported_resources = ("Microsoft.Web/sites/config",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if conf.get('name', '') != 'authsettingsV2':
            return CheckResult.PASSED

        properties = conf.get('properties', {})
        if properties and isinstance(properties, dict):
            platform = properties.get('platform', {})
            if platform and isinstance(properties, dict):
                enabled = platform.get('enabled', False)
                if enabled:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = FunctionAppsEnableAuthentication()



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class DangerousGitSync(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # Use of GITSYNC_GIT has potential for code injection
        # https://www.akamai.com/blog/security-research/2024-august-kubernetes-gitsync-command-injection-defcon
        name = "Limit the use of git-sync to prevent code injection"
        id = "CKV_K8S_159"
        # Location: spec.template.spec.containers[*].env[*].name
        # Location2: spec.template.spec.initContainers[*].env[*].name
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["env"]
        if conf.get("env") and isinstance(conf.get("env"), list):
            for env in conf.get("env", []):
                if env.get("name") and env.get("name") == "GITSYNC_GIT":
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = DangerousGitSync()



from __future__ import annotations
import json
from typing import Any, Dict

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.graph.graph_builder import CustomAttributes


class S3SecureDataTransport(BaseResourceCheck):
    def __init__(self):
        name = "Ensure AWS S3 bucket is configured with secure data transport policy"
        id = "CKV_AWS_379"
        supported_resources = ('aws_s3_bucket_acl',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def _is_policy_secure(self, policy: Dict[str, Any]) -> CheckResult:
        # Explicitly deny aws:SecureTransport = false or allow aws:SecureTransport = true
        if policy.get("Statement"):
            for p in policy.get("Statement"):
                if p.get("Effect") == "Allow":
                    condition = p.get("Condition")
                    if (condition and condition.get("Bool") and
                            'aws:SecureTransport' in condition.get("Bool") and
                            condition.get("Bool").get("aws:SecureTransport").lower() == "true"):
                        return CheckResult.PASSED
                elif p.get("Effect") == "Deny":
                    condition = p.get("Condition")
                    if (condition and condition.get("Bool") and
                            'aws:SecureTransport' in condition.get("Bool") and
                            (not condition.get("Bool").get("aws:SecureTransport") or
                             condition.get("Bool").get("aws:SecureTransport").lower() == "false")):
                        return CheckResult.PASSED
        elif policy.get("statement"):
            policy_statement = policy.get("statement")
            if isinstance(policy_statement, dict):
                policy_statement = [policy_statement]
            for p in policy_statement:
                # Pass if aws:SecureTransport exists
                if ((not p.get("effect") or p.get("effect") == "Allow") and p.get("condition") and
                        p.get("condition").get("test") and p.get("condition").get("test") == "Bool" and
                        p.get("condition").get("variable") and
                        p.get("condition").get("variable") == "aws:SecureTransport" and
                        p.get("condition").get("values") and p.get("condition").get("values")[0]):
                    return CheckResult.PASSED
                elif ((not p.get("effect") or p.get("effect") == "Deny") and p.get("condition") and
                        p.get("condition").get("test") and p.get("condition").get("test") == "Bool" and
                        p.get("condition").get("variable") and
                        p.get("condition").get("variable") == "aws:SecureTransport" and
                        p.get("condition").get("values") and not p.get("condition").get("values")[0]):
                    return CheckResult.PASSED
        return CheckResult.FAILED

    def scan_resource_conf(self, conf) -> CheckResult:
        acl = conf.get('acl')
        is_public = False
        connected_public_access_block = []
        if acl and acl[0] in ('public-read', 'public-read-write'):
            # Search for a connected aws_s3_bucket then a connected aws_s3_bucket_public_access_block then check if
            # restrict_public_buckets is true and pass or else fail
            bucket_id = conf.get("bucket")[0].rsplit('.', 1)[0]
            connected_public_access_block = [
                g for g in self.graph.nodes()
                if g[1].get(CustomAttributes.RESOURCE_TYPE) == "aws_s3_bucket_public_access_block"
                and isinstance(g[1].get("bucket"), str)
                and g[1].get("bucket").rsplit('.', 1)[0] == bucket_id
            ]
            if connected_public_access_block:
                if (not connected_public_access_block[0][1].get('restrict_public_buckets') and
                        not connected_public_access_block[0][1].get('block_public_acls')):
                    is_public = True
            else:
                is_public = True

        access_control_policy = conf.get('access_control_policy')
        if not is_public and access_control_policy:
            grants = access_control_policy[0].get('grant', [])
            for grant in grants:
                grantee = grant.get('grantee', [])
                if grantee and grantee[0].get('uri', [None])[0] == 'http://acs.amazonaws.com/groups/global/AllUsers':
                    # Search for a connected aws_s3_bucket then a connected aws_s3_bucket_public_access_block then
                    # check if block_public_acls is true and pass or else fail
                    bucket_id = conf.get("bucket")[0].rsplit('.', 1)[0]
                    # Don't look again if already collected
                    if not connected_public_access_block:
                        connected_public_access_block = [
                            g for g in self.graph.nodes()
                            if g[1].get(CustomAttributes.RESOURCE_TYPE) == "aws_s3_bucket_public_access_block"
                            and isinstance(g[1].get("bucket"), str)
                            and g[1].get("bucket").rsplit('.', 1)[0] == bucket_id
                        ]
                    if connected_public_access_block:
                        if not connected_public_access_block[0][1].get('block_public_acls'):
                            is_public = True
                    else:
                        is_public = True

        if not is_public:
            return CheckResult.PASSED

        # if connected to aws_s3_bucket_website_configuration then pass
        bucket_id = conf.get("bucket")[0].rsplit('.', 1)[0]
        connected_website = [
            g for g in self.graph.nodes()
            if g[1].get(CustomAttributes.RESOURCE_TYPE) == "aws_s3_bucket_website_configuration"
            and isinstance(g[1].get("bucket"), str)
            and g[1].get("bucket").rsplit('.', 1)[0] == bucket_id
        ]
        if connected_website:
            return CheckResult.PASSED

        # Ensures the aws:SecureTransport condition does not exist in any policy statement.
        connected_s3_bucket_policy = [
            g for g in self.graph.nodes()
            if g[1].get(CustomAttributes.RESOURCE_TYPE) == "aws_s3_bucket_policy"
            and isinstance(g[1].get("bucket"), str)
            and g[1].get("bucket").rsplit('.', 1)[0] == bucket_id
        ]

        if connected_s3_bucket_policy:
            policy_statement = connected_s3_bucket_policy[0][1].get("policy")
            if isinstance(policy_statement, dict):
                return self._is_policy_secure(policy_statement)
            elif isinstance(policy_statement, str) and policy_statement.strip().startswith('jsonencode'):
                json_content = policy_statement.replace("jsonencode(", "").replace(")", "")
                json_content = json_content.replace("'", '"')
                json_content = json_content.replace('""', '"')
                try:
                    policy_statement = json.loads(json_content)
                except json.JSONDecodeError:
                    # Error decoding JSON
                    return CheckResult.UNKNOWN
                return self._is_policy_secure(policy_statement)
            elif (isinstance(policy_statement, str) and policy_statement.split('.')[0] == 'data' and
                  policy_statement.split('.')[-1] == 'json'):
                target_id = '.'.join(policy_statement.split('.')[1:-1])
                connected_iam_policy_doc = [
                    g2 for g2 in self.graph.nodes()
                    if g2[1].get(CustomAttributes.BLOCK_TYPE) == "data"
                    and g2[1].get(CustomAttributes.ID) == target_id
                ]

                if connected_iam_policy_doc[0][1].get("statement"):
                    return self._is_policy_secure(connected_iam_policy_doc[0][1])

        return CheckResult.UNKNOWN


check = S3SecureDataTransport()



from typing import Any, List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import \
    BaseResourceNegativeValueCheck


class CLBListenerProtocol(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud CLBs use modern, encrypted protocols"
        id = "CKV_TC_12"
        supported_resources = ['tencentcloud_clb_listener']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'protocol/[0]'

    def get_forbidden_values(self) -> List[Any]:
        return ["TCP", "UDP", "HTTP"]


check = CLBListenerProtocol()



from dataclasses import dataclass
from typing import Literal

from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType as CommonBlockType

BlockTypeAlias: TypeAlias = Literal["targetScope", "param", "var", "resource", "module", "output"]


@dataclass
class BlockType(CommonBlockType):
    TARGET_SCOPE: Literal["targetScope"] = "targetScope"
    PARAM: Literal["param"] = "param"
    VAR: Literal["var"] = "var"
    MODULE: Literal["module"] = "module"
    OUTPUT: Literal["output"] = "output"



from ipaddress import ip_network, ip_address
from typing import Any, List, Dict

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class VnetLocalDNS(BaseResourceCheck):
    def __init__(self) -> None:
        """Avoid taking a dependency on external DNS servers
                 for local communication such as those deployed on-premises.
                Where possible consider deploying Azure Private DNS Zones,
                 a platform-as-a-service (PaaS) DNS service for VNETs"""

        name = "Ensure that VNET uses local DNS addresses"
        id = "CKV_AZURE_183"
        supported_resources = ("Microsoft.Network/virtualNetworks",)
        categories = [CheckCategories.NETWORKING, ]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Dict[str, Dict[str, List[Any]]]]) -> CheckResult:
        if "properties" in conf and "dhcpOptions" in conf["properties"]:
            if "dnsServers" in conf["properties"]["dhcpOptions"]:
                if isinstance(conf["properties"]["dhcpOptions"]["dnsServers"], list):
                    dns_servers = conf["properties"]["dhcpOptions"]["dnsServers"]
                    if dns_servers:
                        for ip in dns_servers:
                            if "addressSpace" in conf["properties"] and conf["properties"]["addressSpace"]:
                                if "addressPrefixes" in conf["properties"]["addressSpace"]:
                                    if isinstance(conf["properties"]["addressSpace"]["addressPrefixes"], list):
                                        address_spaces = conf["properties"]["addressSpace"]["addressPrefixes"]
                                        if isinstance(address_spaces, list):
                                            for address_range in address_spaces:
                                                if not isinstance(address_range, str):
                                                    continue
                                                try:
                                                    net = ip_network(address_range)
                                                    ip_add = ip_address(ip) if isinstance(ip, str) else None
                                                except ValueError:
                                                    return CheckResult.UNKNOWN
                                                if isinstance(ip, str) and ip_add in net:
                                                    return CheckResult.PASSED
                    self.evaluated_keys = ["dnsServers"]
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = VnetLocalDNS()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import \
    BaseResourceCheck


class CLBInstanceLog(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud CLB has a logging ID and topic"
        id = "CKV_TC_11"
        supported_resources = ['tencentcloud_clb_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("log_set_id") is None or conf.get("log_topic_id") is None:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = CLBInstanceLog()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

ALLOWED_SSL_MODES = ["TRUSTED_CLIENT_CERTIFICATE_REQUIRED"]


class GoogleCloudSqlDatabaseRequireSsl(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all Cloud SQL database instance requires all incoming connections to use SSL"
        id = "CKV_GCP_6"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        Looks for google_sql_database_instance which do not enforce SSL connections:
        :param
        conf: google_sql_database_instance
        configuration
        :return: < CheckResult >
        """

        if 'settings' in conf.keys() and 'ip_configuration' in conf['settings'][0]:
            ipconfiguration = conf['settings'][0]['ip_configuration'][0]

            if 'ssl_mode' in ipconfiguration:
                ssl_mode = ipconfiguration['ssl_mode']
                ssl_mode = ssl_mode[0] if isinstance(ssl_mode, list) else ssl_mode

                if ssl_mode in ALLOWED_SSL_MODES:
                    return CheckResult.PASSED

            elif 'require_ssl' in ipconfiguration:

                require_ssl = ipconfiguration['require_ssl']
                require_ssl = require_ssl[0] if isinstance(require_ssl, list) else require_ssl

                if require_ssl:
                    return CheckResult.PASSED

        return CheckResult.FAILED

    def get_inspected_keys(self):
        return ['settings/[0]/ip_configuration/[0]/ssl_mode/[0]', 'settings/[0]/ip_configuration/[0]/require_ssl/[0]']


check = GoogleCloudSqlDatabaseRequireSsl()



from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class GKEPodSecurityPolicyEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Kubernetes Cluster is created with Private cluster enabled"
        id = "CKV_GCP_25"
        supported_resources = ("google_container_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "private_cluster_config"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = GKEPodSecurityPolicyEnabled()



from typing import Any, List, Dict

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class VnetSingleDNSServer(BaseResourceCheck):

    def __init__(self) -> None:
        """Using a single DNS server may indicate a single point of failure
        where the DNS IP address is not load balanced."""
        name = "Ensure that VNET has at least 2 connected DNS Endpoints"
        id = "CKV_AZURE_182"
        supported_resources = ("Microsoft.Network/networkInterfaces", "Microsoft.Network/virtualNetworks")
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Dict[str, Dict[str, List[Any]]]]) -> CheckResult:
        if "properties" in conf and "dnsSettings" in conf["properties"]:
            if "dnsServers" in conf["properties"]["dnsSettings"] and isinstance(
                    conf["properties"]["dnsSettings"]["dnsServers"], list):
                dns_servers = conf["properties"]["dnsSettings"]["dnsServers"]
                if dns_servers and len(dns_servers) == 1:
                    self.evaluated_keys = ["dnsServers"]
                    return CheckResult.FAILED
        else:
            if "properties" in conf and "dhcpOptions" in conf["properties"]:
                if "dnsServers" in conf["properties"]["dhcpOptions"] and isinstance(
                        conf["properties"]["dhcpOptions"]["dnsServers"], list):
                    dns_servers = conf["properties"]["dhcpOptions"]["dnsServers"]
                    if dns_servers and len(dns_servers) == 1:
                        self.evaluated_keys = ["dnsServers"]
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = VnetSingleDNSServer()
